@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: connect.c,v 1.56 2003/05/13 12:12:17 bagder Exp $
+ * $Id: connect.c,v 1.59 2003/08/11 23:15:13 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -70,6 +70,7 @@
 #define EINPROGRESS WSAEINPROGRESS
 #define EWOULDBLOCK WSAEWOULDBLOCK
 #define EISCONN     WSAEISCONN
+#define ENOTSOCK    WSAENOTSOCK
 #endif
 
 #include "urldata.h"
@@ -77,7 +78,7 @@
 #include "if2ip.h"
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
@@ -190,11 +191,6 @@ int waitconnect(int sockfd, /* socket */
 static CURLcode bindlocal(struct connectdata *conn,
                           int sockfd)
 {
-#if !defined(WIN32)||defined(__CYGWIN32__)
-  /* We don't generally like checking for OS-versions, we should make this
-     HAVE_XXXX based, although at the moment I don't have a decent test for
-     this! */
-
 #ifdef HAVE_INET_NTOA
 
   struct SessionHandle *data = conn->data;
@@ -208,6 +204,7 @@ static CURLcode bindlocal(struct connectdata *conn,
     size_t size;
     char myhost[256] = "";
     in_addr_t in;
+    int rc;
 
     /* First check if the given name is an IP address */
     in=inet_addr(data->set.device);
@@ -217,7 +214,10 @@ static CURLcode bindlocal(struct connectdata *conn,
       /*
        * We now have the numerical IPv4-style x.y.z.w in the 'myhost' buffer
        */
-      h = Curl_resolv(data, myhost, 0);
+      rc = Curl_resolv(conn, myhost, 0, &h);
+      if(rc == 1)
+        rc = Curl_wait_for_resolv(conn, &h);
+
     }
     else {
       if(strlen(data->set.device)>1) {
@@ -225,11 +225,14 @@ static CURLcode bindlocal(struct connectdata *conn,
          * This was not an interface, resolve the name as a host name
          * or IP number
          */
-        h = Curl_resolv(data, data->set.device, 0);
-        if(h) {
+        rc = Curl_resolv(conn, data->set.device, 0, &h);
+        if(rc == 1)
+          rc = Curl_wait_for_resolv(conn, &h);
+
+        if(h)
           /* we know data->set.device is shorter than the myhost array */
           strcpy(myhost, data->set.device);
-        }
+
       }
     }
 
@@ -335,7 +338,6 @@ static CURLcode bindlocal(struct connectdata *conn,
 
   } /* end of device selection support */
 #endif /* end of HAVE_INET_NTOA */
-#endif /* end of not WIN32 */
 
   return CURLE_HTTP_PORT_FAILED;
 }
