@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: hash.c,v 1.12 2002/11/05 10:51:44 bagder Exp $
+ * $Id: hash.c,v 1.16 2003/08/14 15:05:13 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -29,14 +29,12 @@
 #include "hash.h"
 #include "llist.h"
 
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 /* this must be the last include file */
 #include "memdebug.h"
 #endif
 
 
-/* {{{ static unsigned long _hash_str (const char *, size_t)
- */
 static unsigned long
 _hash_str (const char *key, size_t key_length)
 {
@@ -50,10 +48,7 @@ _hash_str (const char *key, size_t key_length)
 
   return h;
 }
-/* }}} */
 
-/* {{{ static void _hash_element_dtor (void *, void *)
- */
 static void 
 _hash_element_dtor (void *user, void *element)
 {
@@ -68,12 +63,10 @@ _hash_element_dtor (void *user, void *element)
 
   free(e);
 }
-/* }}} */
 
-/* {{{ void curl_hash_init (curl_hash *, int, curl_hash_dtor)
- */
-void 
-Curl_hash_init (curl_hash *h, int slots, curl_hash_dtor dtor)
+/* return 1 on error, 0 is fine */
+int
+Curl_hash_init(curl_hash *h, int slots, curl_hash_dtor dtor)
 {
   int i;
 
@@ -82,31 +75,39 @@ Curl_hash_init (curl_hash *h, int slots, curl_hash_dtor dtor)
   h->slots = slots;  
 
   h->table = (curl_llist **) malloc(slots * sizeof(curl_llist *));
-  for (i = 0; i < slots; ++i) {
-    h->table[i] = Curl_llist_alloc((curl_llist_dtor) _hash_element_dtor);
+  if(h->table) {
+    for (i = 0; i < slots; ++i) {
+      h->table[i] = Curl_llist_alloc((curl_llist_dtor) _hash_element_dtor);
+      if(!h->table[i]) {
+        while(i--)
+          Curl_llist_destroy(h->table[i], NULL);
+        free(h->table);
+        return 1; /* failure */
+      }
+    }
+    return 0; /* fine */
   }
+  else
+    return 1; /* failure */
 }
-/* }}} */
 
-/* {{{ curl_hash *curl_hash_alloc (int, curl_hash_dtor)
- */
 curl_hash *
-Curl_hash_alloc (int slots, curl_hash_dtor dtor)
+Curl_hash_alloc(int slots, curl_hash_dtor dtor)
 {
   curl_hash *h;
 
   h = (curl_hash *) malloc(sizeof(curl_hash));
-  if (NULL == h)
-    return NULL;
-
-  Curl_hash_init(h, slots, dtor);
+  if (h) {
+    if(Curl_hash_init(h, slots, dtor)) {
+      /* failure */
+      free(h);
+      h = NULL;
+    }
+  }
 
   return h;
 }
-/* }}} */
 
-/* {{{ static int _hash_key_compare (char *, size_t, char *, size_t)
- */
 static int 
 _hash_key_compare (char *key1, size_t key1_len, char *key2, size_t key2_len)
 {
@@ -118,12 +119,10 @@ _hash_key_compare (char *key1, size_t key1_len, char *key2, size_t key2_len)
 
   return 0;
 }
-/* }}} */
 
-/* {{{ static int _mk_hash_element (curl_hash_element **, char *, size_t, const void *)
- */
 static int
-_mk_hash_element (curl_hash_element **e, char *key, size_t key_len, const void *p)
+_mk_hash_element (curl_hash_element **e, char *key, size_t key_len,
+                  const void *p)
 {
   *e = (curl_hash_element *) malloc(sizeof(curl_hash_element));
   (*e)->key = strdup(key);
@@ -131,16 +130,12 @@ _mk_hash_element (curl_hash_element **e, char *key, size_t key_len, const void *
   (*e)->ptr = (void *) p;
   return 0;
 }
-/* }}} */
 
 #define find_slot(__h, __k, __k_len) (_hash_str(__k, __k_len) % (__h)->slots)
 
 #define FETCH_LIST \
   curl_llist *l = h->table[find_slot(h, key, key_len)]
 
-
-/* {{{ int curl_hash_add (curl_hash *, char *, size_t, const void *)
- */
 int 
 Curl_hash_add (curl_hash *h, char *key, size_t key_len, const void *p)
 {
@@ -169,10 +164,7 @@ Curl_hash_add (curl_hash *h, char *key, size_t key_len, const void *p)
 
   return 0;
 }
-/* }}} */
 
-/* {{{ int curl_hash_delete (curl_hash *, char *, size_t)
- */
 int 
 Curl_hash_delete(curl_hash *h, char *key, size_t key_len)
 {
@@ -193,10 +185,7 @@ Curl_hash_delete(curl_hash *h, char *key, size_t key_len)
 
   return 0;
 }
-/* }}} */
 
-/* {{{ int curl_hash_pick (curl_hash *, char *, size_t, void **)
- */
 void *
 Curl_hash_pick(curl_hash *h, char *key, size_t key_len)
 {
@@ -215,10 +204,7 @@ Curl_hash_pick(curl_hash *h, char *key, size_t key_len)
 
   return NULL;
 }
-/* }}} */
 
-/* {{{ void curl_hash_apply (curl_hash *, void *, void (*)(void *, curl_hash_element *))
- */
 void 
 Curl_hash_apply(curl_hash *h, void *user,
                 void (*cb)(void *user, void *ptr))
@@ -235,10 +221,7 @@ Curl_hash_apply(curl_hash *h, void *user,
     }
   }
 }
-/* }}} */
 
-/* {{{ void curl_hash_clean (curl_hash *)
- */
 void
 Curl_hash_clean(curl_hash *h)
 {
@@ -250,11 +233,7 @@ Curl_hash_clean(curl_hash *h)
 
   free(h->table);
 }
-/* }}} */
 
-/* {{{ void curl_hash_clean_with_criterium (curl_hash *, void *,
-   int (*)(void *, void *))
- */
 void
 Curl_hash_clean_with_criterium(curl_hash *h, void *user,
                                int (*comp)(void *, void *))
@@ -277,17 +256,12 @@ Curl_hash_clean_with_criterium(curl_hash *h, void *user,
   }
 }
 
-/* {{{ int curl_hash_count (curl_hash *)
- */
-int 
+int
 Curl_hash_count(curl_hash *h)
 {
   return h->size;
 }
-/* }}} */
 
-/* {{{ void curl_hash_destroy (curl_hash *)
- */
 void 
 Curl_hash_destroy(curl_hash *h)
 {
@@ -297,12 +271,4 @@ Curl_hash_destroy(curl_hash *h)
   Curl_hash_clean(h);
   free(h);
 }
-/* }}} */
 
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
