@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http.c,v 1.117 2002/12/10 13:10:00 bagder Exp $
+ * $Id: http.c,v 1.151 2003/08/12 08:20:16 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -54,7 +54,6 @@
 #endif
 #endif
 
-#include <sys/resource.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -89,15 +88,23 @@
 #include "cookie.h"
 #include "strequal.h"
 #include "ssluse.h"
+#include "http_digest.h"
+#include "http_ntlm.h"
+#include "http_negotiate.h"
+#include "url.h"
+#include "share.h"
+#include "http.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
 #include <curl/mprintf.h>
 
 /* The last #include file should be: */
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 #include "memdebug.h"
 #endif
 
+static CURLcode Curl_output_basic_proxy(struct connectdata *conn);
+
 /* fread() emulation to provide POST and/or request data */
 static int readmoredata(char *buffer,
                         size_t size,
@@ -190,6 +197,7 @@ CURLcode add_buffer_send(send_buffer *in,
   char *ptr;
   int size;
   struct HTTP *http = conn->proto.http;
+  int sendsize;
 
   /* The looping below is required since we use non-blocking sockets, but due
      to the circumstances we will just loop and try again and again etc */
@@ -197,7 +205,28 @@ CURLcode add_buffer_send(send_buffer *in,
   ptr = in->buffer;
   size = in->size_used;
 
-  res = Curl_write(conn, sockfd, ptr, size, &amount);
+  if(conn->protocol & PROT_HTTPS) {
+    /* We never send more than CURL_MAX_WRITE_SIZE bytes in one single chunk
+       when we speak HTTPS, as if only a fraction of it is sent now, this data
+       needs to fit into the normal read-callback buffer later on and that
+       buffer is using this size.
+    */
+
+    sendsize= (size > CURL_MAX_WRITE_SIZE)?CURL_MAX_WRITE_SIZE:size;
+
+    /* OpenSSL is very picky and we must send the SAME buffer pointer to the
+       library when we attempt to re-send this buffer. Sending the same data
+       is not enough, we must use the exact same address. For this reason, we
+       must copy the data to the uploadbuffer first, since that is the buffer
+       we will be using if this send is retried later.
+    */
+    memcpy(conn->data->state.uploadbuffer, ptr, sendsize);
+    ptr = conn->data->state.uploadbuffer;
+  }
+  else
+    sendsize = size;
+  
+  res = Curl_write(conn, sockfd, ptr, sendsize, &amount);
 
   if(CURLE_OK == res) {
 
@@ -213,7 +242,8 @@ CURLcode add_buffer_send(send_buffer *in,
          and wait until it might work again. */
 
       size -= amount;
-      ptr += amount;
+
+      ptr = in->buffer + amount;
     
       /* backup the currently set pointers */
       http->backup.fread = conn->fread;
@@ -232,7 +262,7 @@ CURLcode add_buffer_send(send_buffer *in,
       
       return CURLE_OK;
     }
-
+    http->sending = HTTPSEND_BODY;
     /* the full buffer was sent, clean up and return */
   }
   if(in->buffer)
@@ -403,6 +433,13 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
 
   infof(data, "Establish HTTP proxy tunnel to %s:%d\n", hostname, remote_port);
 
+  /*
+   * This code currently only supports Basic authentication for this CONNECT
+   * request to a proxy.
+   */
+  if(conn->bits.proxy_user_passwd)
+    Curl_output_basic_proxy(conn);
+
   /* OK, now send the connect request to the proxy */
   result =
     Curl_sendf(tunnelsocket, conn,
@@ -491,16 +528,16 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
             /* a newline is CRLF in ftp-talk, so the CR is ignored as
                the line isn't really terminated until the LF comes */
 
-            /* output debug output if that is requested */
-            if(data->set.verbose)
-              Curl_debug(data, CURLINFO_DATA_IN, line_start, perline);
-            
             if('\r' == line_start[0]) {
               /* end of headers */
               keepon=FALSE;
               break; /* breaks out of loop, not switch */
             }
 
+            /* output debug output if that is requested */
+            if(data->set.verbose)
+              Curl_debug(data, CURLINFO_HEADER_IN, line_start, perline);
+
             if(2 == sscanf(line_start, "HTTP/1.%d %d",
                            &subversion,
                            &httperror)) {
@@ -519,6 +556,8 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
   if(error)
     return CURLE_RECV_ERROR;
 
+  data->info.httpproxycode = httperror;
+
   if(200 != httperror) {
     if(407 == httperror)
       /* Added Nov 6 1998 */
@@ -527,6 +566,14 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
       failf(data, "Received error code %d from proxy", httperror);
     return CURLE_RECV_ERROR;
   }
+  
+  /* If a proxy-authorization header was used for the proxy, then we should
+     make sure that it isn't accidentally used for the document request
+     after we've connected. So let's free and clear it here. */
+  Curl_safefree(conn->allocptr.proxyuserpwd);
+  conn->allocptr.proxyuserpwd = NULL;
+
+  Curl_http_auth_stage(data, 401); /* move on to the host auth */
 
   infof (data, "Proxy replied to CONNECT request\n");
   return CURLE_OK;
@@ -591,11 +638,15 @@ CURLcode Curl_http_done(struct connectdata *conn)
   conn->fread = data->set.fread; /* restore */
   conn->fread_in = data->set.in; /* restore */
 
+  if (http == NULL) 
+    return CURLE_OK;
+
   if(http->send_buffer) {
     send_buffer *buff = http->send_buffer;
     
     free(buff->buffer);
     free(buff);
+    http->send_buffer = NULL; /* cleaer the pointer */
   }
 
   if(HTTPREQ_POST_FORM == data->set.httpreq) {
@@ -616,6 +667,56 @@ CURLcode Curl_http_done(struct connectdata *conn)
   return CURLE_OK;
 }
 
+static CURLcode Curl_output_basic(struct connectdata *conn)
+{
+  char *authorization;
+  struct SessionHandle *data=conn->data;
+
+  sprintf(data->state.buffer, "%s:%s", conn->user, conn->passwd);
+  if(Curl_base64_encode(data->state.buffer, strlen(data->state.buffer),
+                        &authorization) >= 0) {
+    if(conn->allocptr.userpwd)
+      free(conn->allocptr.userpwd);
+    conn->allocptr.userpwd = aprintf( "Authorization: Basic %s\015\012",
+                                      authorization);
+    free(authorization);
+  }
+  else
+    return CURLE_OUT_OF_MEMORY;
+  return CURLE_OK;
+}
+
+static CURLcode Curl_output_basic_proxy(struct connectdata *conn)
+{
+  char *authorization;
+  struct SessionHandle *data=conn->data;
+
+  sprintf(data->state.buffer, "%s:%s", conn->proxyuser, conn->proxypasswd);
+  if(Curl_base64_encode(data->state.buffer, strlen(data->state.buffer),
+                        &authorization) >= 0) {
+    Curl_safefree(conn->allocptr.proxyuserpwd);
+    conn->allocptr.proxyuserpwd =
+      aprintf("Proxy-authorization: Basic %s\015\012", authorization);
+    free(authorization);
+  }
+  else
+    return CURLE_OUT_OF_MEMORY;
+  return CURLE_OK;
+}
+
+void Curl_http_auth_stage(struct SessionHandle *data,
+                          int stage)
+{
+  if(stage == 401)
+    data->state.authwant = data->set.httpauth;
+  else if(stage == 407)
+    data->state.authwant = data->set.proxyauth;
+  else
+    return; /* bad input stage */
+  data->state.authstage = stage;
+  data->state.authavail = CURLAUTH_NONE;
+}
+
 CURLcode Curl_http(struct connectdata *conn)
 {
   struct SessionHandle *data=conn->data;
@@ -626,6 +727,15 @@ CURLcode Curl_http(struct connectdata *conn)
   char *ppath = conn->ppath; /* three previous function arguments */
   char *host = conn->name;
   const char *te = ""; /* tranfer-encoding */
+  char *ptr;
+  char *request;
+
+  if(!data->state.authstage) {
+    if(conn->bits.httpproxy && conn->bits.proxy_user_passwd)
+      Curl_http_auth_stage(data, 407);
+    else
+      Curl_http_auth_stage(data, 401);
+  }
 
   if(!conn->proto.http) {
     /* Only allocate this struct if we don't already have it! */
@@ -646,6 +756,13 @@ CURLcode Curl_http(struct connectdata *conn)
        data->set.upload) {
     data->set.httpreq = HTTPREQ_PUT;
   }
+
+  request = data->set.customrequest?
+    data->set.customrequest:
+    (data->set.no_body?(char *)"HEAD":
+     ((HTTPREQ_POST == data->set.httpreq) ||
+      (HTTPREQ_POST_FORM == data->set.httpreq))?(char *)"POST":
+     (HTTPREQ_PUT == data->set.httpreq)?(char *)"PUT":(char *)"GET");
   
   /* The User-Agent string has been built in url.c already, because it might
      have been used in the proxy connect, but if we have got a header with
@@ -656,26 +773,73 @@ CURLcode Curl_http(struct connectdata *conn)
     conn->allocptr.uagent=NULL;
   }
 
-  if((conn->bits.user_passwd) && !checkheaders(data, "Authorization:")) {
-    char *authorization;
+  /* To prevent the user+password to get sent to other than the original
+     host due to a location-follow, we do some weirdo checks here */
+  if(!data->state.this_is_a_follow ||
+     !data->state.auth_host ||
+     curl_strequal(data->state.auth_host, conn->hostname) ||
+     data->set.http_disable_hostname_check_before_authentication) {
 
-    /* To prevent the user+password to get sent to other than the original
-       host due to a location-follow, we do some weirdo checks here */
-    if(!data->state.this_is_a_follow ||
-       !data->state.auth_host ||
-       curl_strequal(data->state.auth_host, conn->hostname)) {
-      sprintf(data->state.buffer, "%s:%s",
-              data->state.user, data->state.passwd);
-      if(Curl_base64_encode(data->state.buffer, strlen(data->state.buffer),
-                            &authorization) >= 0) {
-        if(conn->allocptr.userpwd)
-          free(conn->allocptr.userpwd);
-        conn->allocptr.userpwd = aprintf( "Authorization: Basic %s\015\012",
-                                          authorization);
-        free(authorization);
+  /* Send proxy authentication header if needed */
+    if (data->state.authstage == 407) {
+#ifdef USE_SSLEAY
+      if(data->state.authwant == CURLAUTH_NTLM) {
+        result = Curl_output_ntlm(conn, TRUE);
+        if(result)
+          return result;
+      }
+      else
+#endif
+      if((data->state.authwant == CURLAUTH_BASIC) && /* Basic */
+         conn->bits.proxy_user_passwd &&
+         !checkheaders(data, "Proxy-authorization:")) {
+        result = Curl_output_basic_proxy(conn);
+        if(result)
+          return result;
+        /* Switch to web authentication after proxy authentication is done */
+        Curl_http_auth_stage(data, 401);
+      }
+    }
+    /* Send web authentication header if needed */
+    if (data->state.authstage == 401) {
+#ifdef GSSAPI
+      if((data->state.authwant == CURLAUTH_GSSNEGOTIATE) &&
+         data->state.negotiate.context && 
+         !GSS_ERROR(data->state.negotiate.status)) {
+        result = Curl_output_negotiate(conn);
+        if (result)
+          return result;
+      }
+      else
+#endif
+#ifdef USE_SSLEAY
+      if(data->state.authwant == CURLAUTH_NTLM) {
+        result = Curl_output_ntlm(conn, FALSE);
+        if(result)
+          return result;
+      }
+      else
+#endif
+      {
+        if((data->state.authwant == CURLAUTH_DIGEST) &&
+           data->state.digest.nonce) {
+          result = Curl_output_digest(conn,
+                                      (unsigned char *)request,
+                                      (unsigned char *)ppath);
+          if(result)
+            return result;
+        }
+        else if((data->state.authwant == CURLAUTH_BASIC) && /* Basic */
+                conn->bits.user_passwd &&
+                !checkheaders(data, "Authorization:")) {
+          result = Curl_output_basic(conn);
+          if(result)
+            return result;
+        }
       }
     }
   }
+
   if((data->change.referer) && !checkheaders(data, "Referer:")) {
     if(conn->allocptr.ref)
       free(conn->allocptr.ref);
@@ -713,30 +877,30 @@ CURLcode Curl_http(struct connectdata *conn)
     }
   }
 
-  if(data->cookies) {
-    co = Curl_cookie_getlist(data->cookies,
-                             host, ppath,
-                             (conn->protocol&PROT_HTTPS?TRUE:FALSE));
-  }
-  if (data->change.proxy && *data->change.proxy &&
-      !data->set.tunnel_thru_httpproxy &&
-      !(conn->protocol&PROT_HTTPS))  {
-    /* The path sent to the proxy is in fact the entire URL */
-    ppath = data->change.url;
-  }
-  if(HTTPREQ_POST_FORM == data->set.httpreq) {
-    /* we must build the whole darned post sequence first, so that we have
-       a size of the whole shebang before we start to send it */
-     result = Curl_getFormData(&http->sendit, data->set.httppost,
-                               &http->postsize);
-     if(CURLE_OK != result) {
-       /* Curl_getFormData() doesn't use failf() */
-       failf(data, "failed creating formpost data");
-       return result;
-     }
-  }
+  ptr = checkheaders(data, "Host:");
+  if(ptr) {
+    /* If we have a given custom Host: header, we extract the host name
+       in order to possibly use it for cookie reasons later on. */
+    char *start = ptr+strlen("Host:");
+    char *ptr;
+    while(*start && isspace((int)*start ))
+      start++;
+    ptr = start; /* start host-scanning here */
 
-  if(!checkheaders(data, "Host:")) {
+    /* scan through the string to find the end (space or colon) */
+    while(*ptr && !isspace((int)*ptr) && !(':'==*ptr))
+      ptr++;
+    
+    if(ptr != start) {
+      int len=ptr-start;
+      conn->allocptr.cookiehost = malloc(len+1);
+      if(!conn->allocptr.cookiehost)
+        return CURLE_OUT_OF_MEMORY;
+      memcpy(conn->allocptr.cookiehost, start, len);
+      conn->allocptr.cookiehost[len]=0;
+    }
+  }    
+  else {
     /* if ptr_host is already set, it is almost OK since we only re-use
        connections to the very same host and port, but when we use a HTTP
        proxy we have a persistant connect and yet we must change the Host:
@@ -764,6 +928,34 @@ CURLcode Curl_http(struct connectdata *conn)
                                     conn->remote_port);
   }
 
+  if(data->cookies) {
+    Curl_share_lock(data, CURL_LOCK_DATA_COOKIE, CURL_LOCK_ACCESS_SINGLE);
+    co = Curl_cookie_getlist(data->cookies,
+                             conn->allocptr.cookiehost?
+                             conn->allocptr.cookiehost:host, ppath,
+                             (bool)(conn->protocol&PROT_HTTPS?TRUE:FALSE));
+    Curl_share_unlock(data, CURL_LOCK_DATA_COOKIE);
+  }
+
+  if (conn->bits.httpproxy &&
+      !data->set.tunnel_thru_httpproxy &&
+      !(conn->protocol&PROT_HTTPS))  {
+    /* The path sent to the proxy is in fact the entire URL */
+    ppath = data->change.url;
+  }
+  if(HTTPREQ_POST_FORM == data->set.httpreq) {
+    /* we must build the whole darned post sequence first, so that we have
+       a size of the whole shebang before we start to send it */
+     result = Curl_getFormData(&http->sendit, data->set.httppost,
+                               &http->postsize);
+     if(CURLE_OK != result) {
+       /* Curl_getFormData() doesn't use failf() */
+       failf(data, "failed creating formpost data");
+       return result;
+     }
+  }
+
+
   if(!checkheaders(data, "Pragma:"))
     http->p_pragma = "Pragma: no-cache\r\n";
 
@@ -860,13 +1052,14 @@ CURLcode Curl_http(struct connectdata *conn)
     }
   }
 
-  do {
+  {
     /* Use 1.1 unless the use specificly asked for 1.0 */
     const char *httpstring=
       data->set.httpversion==CURL_HTTP_VERSION_1_0?"1.0":"1.1";
 
     send_buffer *req_buffer;
     struct curl_slist *headers=data->set.headers;
+    size_t postsize;
 
     /* initialize a dynamic send-buffer */
     req_buffer = add_buffer_init();
@@ -874,7 +1067,7 @@ CURLcode Curl_http(struct connectdata *conn)
     /* add the main request stuff */
     add_bufferf(req_buffer,
                 "%s " /* GET/HEAD/POST/PUT */
-                "%s HTTP/%s\r\n" /* path */
+                "%s HTTP/%s\r\n" /* path + HTTP version */
                 "%s" /* proxyuserpwd */
                 "%s" /* userpwd */
                 "%s" /* range */
@@ -887,16 +1080,12 @@ CURLcode Curl_http(struct connectdata *conn)
                 "%s" /* referer */
                 "%s",/* transfer-encoding */
 
-                data->set.customrequest?data->set.customrequest:
-                (data->set.no_body?"HEAD":
-                 ((HTTPREQ_POST == data->set.httpreq) ||
-                  (HTTPREQ_POST_FORM == data->set.httpreq))?"POST":
-                 (HTTPREQ_PUT == data->set.httpreq)?"PUT":"GET"),
-                ppath, httpstring,
-                (conn->bits.proxy_user_passwd &&
-                 conn->allocptr.proxyuserpwd)?conn->allocptr.proxyuserpwd:"",
-                (conn->bits.user_passwd && conn->allocptr.userpwd)?
-                conn->allocptr.userpwd:"",
+                request,
+                ppath,
+                httpstring,
+                (conn->bits.httpproxy && conn->allocptr.proxyuserpwd)?
+                conn->allocptr.proxyuserpwd:"",
+                conn->allocptr.userpwd?conn->allocptr.userpwd:"",
                 (conn->bits.use_range && conn->allocptr.rangeline)?
                 conn->allocptr.rangeline:"",
                 (data->set.useragent && *data->set.useragent && conn->allocptr.uagent)?
@@ -916,7 +1105,7 @@ CURLcode Curl_http(struct connectdata *conn)
       struct Cookie *store=co;
       /* now loop through all cookies that matched */
       while(co) {
-        if(co->value && strlen(co->value)) {
+        if(co->value) {
           if(0 == count) {
             add_bufferf(req_buffer, "Cookie: ");
           }
@@ -997,6 +1186,9 @@ CURLcode Curl_http(struct connectdata *conn)
       headers = headers->next;
     }
 
+    http->postdata = NULL;  /* nothing to post at this point */
+    Curl_pgrsSetUploadSize(data, 0); /* upload size is 0 atm */
+
     switch(data->set.httpreq) {
 
     case HTTPREQ_POST_FORM:
@@ -1101,6 +1293,11 @@ CURLcode Curl_http(struct connectdata *conn)
     case HTTPREQ_POST:
       /* this is the simple POST, using x-www-form-urlencoded style */
 
+      /* store the size of the postfields */
+      postsize = data->set.postfieldsize?
+        data->set.postfieldsize:
+        (data->set.postfields?strlen(data->set.postfields):0);
+      
       if(!conn->bits.upload_chunky) {
         /* We only set Content-Length and allow a custom Content-Length if
            we don't upload data chunked, as RFC2616 forbids us to set both
@@ -1109,11 +1306,7 @@ CURLcode Curl_http(struct connectdata *conn)
         if(!checkheaders(data, "Content-Length:"))
           /* we allow replacing this header, although it isn't very wise to
              actually set your own */
-          add_bufferf(req_buffer,
-                      "Content-Length: %d\r\n",
-                      data->set.postfieldsize?
-                      data->set.postfieldsize:
-                      (data->set.postfields?strlen(data->set.postfields):0) );
+          add_bufferf(req_buffer, "Content-Length: %d\r\n", postsize);
       }
 
       if(!checkheaders(data, "Content-Type:"))
@@ -1122,21 +1315,38 @@ CURLcode Curl_http(struct connectdata *conn)
 
       add_buffer(req_buffer, "\r\n", 2);
 
-      /* and here we setup the pointers to the actual data */
       if(data->set.postfields) {
-        if(data->set.postfieldsize)
-          http->postsize = data->set.postfieldsize;
-        else
-          http->postsize = strlen(data->set.postfields);
-        http->postdata = data->set.postfields;
 
-        http->sending = HTTPSEND_BODY;
+        if(postsize < (100*1024)) {
+          /* The post data is less than 100K, then append it to the header.
+             This limit is no magic limit but only set to prevent really huge
+             POSTs to get the data duplicated with malloc() and family. */
 
-        conn->fread = (curl_read_callback)readmoredata;
-        conn->fread_in = (void *)conn;
+          if(!conn->bits.upload_chunky)
+            /* We're not sending it 'chunked', append it to the request
+               already now to reduce the number if send() calls */
+            add_buffer(req_buffer, data->set.postfields, postsize);
+          else {
+            /* Append the POST data chunky-style */
+            add_bufferf(req_buffer, "%x\r\n", postsize);
+            add_buffer(req_buffer, data->set.postfields, postsize);
+            add_buffer(req_buffer, "\r\n0\r\n", 5); /* end of a chunked
+                                                       transfer stream */
+          }
+        }
+        else {
+          /* A huge POST coming up, do data separate from the request */
+          http->postsize = postsize;
+          http->postdata = data->set.postfields;
 
-        /* set the upload size to the progress meter */
-        Curl_pgrsSetUploadSize(data, http->postsize);
+          http->sending = HTTPSEND_BODY;
+
+          conn->fread = (curl_read_callback)readmoredata;
+          conn->fread_in = (void *)conn;
+
+          /* set the upload size to the progress meter */
+          Curl_pgrsSetUploadSize(data, http->postsize);
+        }
       }
       else
         /* set the upload size to the progress meter */
@@ -1152,8 +1362,8 @@ CURLcode Curl_http(struct connectdata *conn)
         result =
           Curl_Transfer(conn, conn->firstsocket, -1, TRUE,
                         &http->readbytecount,
-                        conn->firstsocket,
-                        &http->writebytecount);
+                        http->postdata?conn->firstsocket:-1,
+                        http->postdata?&http->writebytecount:NULL);
       break;
 
     default:
@@ -1174,18 +1384,8 @@ CURLcode Curl_http(struct connectdata *conn)
     }
     if(result)
       return result;
-  } while (0); /* this is just a left-over from the multiple document download
-                  attempts */
+  }
 
   return CURLE_OK;
 }
-
-
-/*
- * local variables:
- * eval: (load-file "../curl-mode.el")
- * end:
- * vim600: fdm=marker
- * vim: et sw=2 ts=2 sts=2 tw=78
- */
 #endif
