@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2002, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2003, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: llist.c,v 1.6 2002/09/03 11:53:00 bagder Exp $
+ * $Id: llist.c,v 1.9 2003/08/14 15:06:08 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -28,7 +28,7 @@
 
 #include "llist.h"
 
-#ifdef MALLOCDEBUG
+#ifdef CURLDEBUG
 /* this must be the last include file */
 #include "memdebug.h"
 #endif
@@ -159,10 +159,10 @@ Curl_llist_count(curl_llist *list)
 void 
 Curl_llist_destroy(curl_llist *list, void *user)
 {
-  while (list->size > 0) {
-    Curl_llist_remove(list, CURL_LLIST_TAIL(list), user);
-  }
+  if(list) {
+    while (list->size > 0)
+      Curl_llist_remove(list, CURL_LLIST_TAIL(list), user);
 
-  free(list);
-  list = NULL;
+    free(list);
+  }
 }
