@@ -20,7 +20,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: setup.h,v 1.17 2003/04/15 14:01:57 bagder Exp $
+ * $Id: setup.h,v 1.20 2003/07/29 11:07:38 bagder Exp $
  ***************************************************************************/
 
 #if !defined(WIN32) && defined(__WIN32__)
@@ -48,8 +48,8 @@
 #endif
 #endif
 
-#ifdef MALLOCDEBUG
-/* This is an ugly hack for MALLOCDEBUG conditions only. We need to include
+#ifdef CURLDEBUG
+/* This is an ugly hack for CURLDEBUG conditions only. We need to include
    the file here, since it might set the _FILE_OFFSET_BITS define, which must
    be set BEFORE all normal system headers. */
 #include "../lib/setup.h"
@@ -66,7 +66,6 @@ int fileno( FILE *stream);
 #endif
 
 #ifdef WIN32
-#define PATH_CHAR     ";"
 #define DIR_CHAR      "\\"
 #define DOT_CHAR      "_"
 #else
@@ -74,17 +73,25 @@ int fileno( FILE *stream);
 /* 20000318 mgs
  * OS/2 supports leading dots in filenames if the volume is formatted
  * with JFS or HPFS. */
-#define PATH_CHAR     ";"
 #define DIR_CHAR      "\\"
 #define DOT_CHAR      "."
 #else
 
-#define PATH_CHAR     ":"
+#ifdef DJGPP
+#include <tcp.h>
+#ifdef word
+#undef word
+#endif
+#define DIR_CHAR      "/"
+#define DOT_CHAR      "_"
+#else
+
 #define DIR_CHAR      "/"
 #define DOT_CHAR      "."
 
-#endif
-#endif
+#endif /* !DJGPP */
+#endif /* !__EMX__ */
+#endif /* !WIN32 */
 
 #ifdef __riscos__
 #define USE_ENVIRONMENT
