@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: ftpserver.pl,v 1.32 2003/04/01 08:42:14 bagder Exp $
+# $Id: ftpserver.pl,v 1.36 2003/08/08 10:21:47 bagder Exp $
 # This is the FTP server designed for the curl test suite.
 #
 # It is meant to exercise curl, it is not meant to be a fully working
@@ -23,7 +23,19 @@ open(FTPLOG, ">log/ftpd.log") ||
 
 sub logmsg { print FTPLOG "$$: "; print FTPLOG @_; }
 
-sub ftpmsg { print INPUT @_; }
+sub ftpmsg {
+  # append to the server.input file
+  open(INPUT, ">>log/server.input") ||
+    logmsg "failed to open log/server.input\n";
+
+  INPUT->autoflush(1);
+  print INPUT @_;
+  close(INPUT);
+
+  # use this, open->print->close system only to make the file
+  # open as little as possible, to make the test suite run
+  # better on windows/cygwin
+}
 
 my $verbose=0; # set to 1 for debugging
 my $retrweirdo=0;
@@ -80,10 +92,12 @@ my %commandok = (
                  'SYST' => 'loggedin',
                  'SIZE' => 'loggedin|twosock',
                  'PWD'  => 'loggedin|twosock',
+                 'MKD'  => 'loggedin|twosock',
                  'QUIT'  => 'loggedin|twosock',
                  'RNFR'  => 'loggedin|twosock',
                  'RNTO'  => 'loggedin|twosock',
-                 'DELE' => 'loggedin|twosock'
+                 'DELE' => 'loggedin|twosock',
+                 'MDTM' => 'loggedin|twosock',
                  );
 
 # initially, we're in 'fresh' state
@@ -105,6 +119,7 @@ my %displaytext = ('USER' => '331 We are happy you popped in!',
                    'SYST' => '215 UNIX Type: L8', # just fake something
                    'QUIT' => '221 bye bye baby', # just reply something
                    'PWD'  => '257 "/nowhere/anywhere" is current directory',
+                   'MKD'  => '257 Created your requested directory',
                    'REST' => '350 Yeah yeah we set it there for you',
                    'DELE' => '200 OK OK OK whatever you say',
                    'RNFR' => '350 Received your order. Please provide more',
@@ -122,6 +137,7 @@ my %commandfunc = ( 'PORT' => \&PORT_command,
                     'REST' => \&REST_command,
                     'STOR' => \&STOR_command,
                     'APPE' => \&STOR_command, # append looks like upload
+                    'MDTM' => \&MDTM_command,
                     );
 
 my $rest=0;
@@ -167,6 +183,33 @@ sub NLST_command {
     return 0;
 }
 
+sub MDTM_command {
+    my $testno = $_[0];
+
+    loadtest("data/test$testno");
+
+    logmsg "MDTM $testno\n";
+
+    my @data = getpart("reply", "mdtm");
+
+    my $reply = $data[0];
+    chomp $reply;
+
+    if($reply <0) {
+        print "550 $testno: no such file.\r\n";
+        logmsg "MDTM $testno: no such file\n";
+    }
+    elsif($reply) {
+        print "$reply\r\n";
+        logmsg "MDTM $testno returned $reply\n";
+    }
+    else {
+        print "500 MDTM: no such command.\r\n";
+        logmsg "MDTM: no such command\n";
+    }
+    return 0;
+}
+
 sub SIZE_command {
     my $testno = $_[0];
 
@@ -387,6 +430,7 @@ sub PORT_command {
 $SIG{CHLD} = \&REAPER;
 
 my %customreply;
+my %customcount;
 my %delayreply;
 sub customize {
     undef %customreply;
@@ -399,6 +443,11 @@ sub customize {
         if($_ =~ /REPLY ([A-Z]+) (.*)/) {
             $customreply{$1}=$2;
         }
+        if($_ =~ /COUNT ([A-Z]+) (.*)/) {
+            # we blank the customreply for this command when having
+            # been used this number of times
+            $customcount{$1}=$2;
+        }
         elsif($_ =~ /DELAY ([A-Z]+) (\d*)/) {
             $delayreply{$1}=$2;
         }
@@ -437,12 +486,7 @@ for ( $waitedpid = 0;
     open(STDIN,  "<&Client")   || die "can't dup client to stdin";
     open(STDOUT, ">&Client")   || die "can't dup client to stdout";
     
-    open(INPUT, ">log/server.input") ||
-        logmsg "failed to open log/server.input\n";
-
     FTPLOG->autoflush(1);
-    INPUT->autoflush(1);
-
     &customize(); # read test control instructions
 
     print @welcome;
@@ -504,6 +548,10 @@ for ( $waitedpid = 0;
             $text = $displaytext{$FTPCMD};
         }
         else {
+            if($customcount{$FTPCMD} && (!--$customcount{$FTPCMD})) {
+                # used enough number of times, now blank the customreply
+                $customreply{$FTPCMD}="";
+            }
             logmsg "$FTPCMD made to send '$text'\n";
         }
         if($text) {
