@@ -1,5 +1,5 @@
 #!/usr/bin/env perl
-# $Id: runtests.pl,v 1.80 2003/04/30 20:28:49 bagder Exp $
+# $Id: runtests.pl,v 1.91 2003/08/14 12:59:54 bagder Exp $
 #
 # Main curl test script, in perl to run on more platforms
 #
@@ -48,13 +48,8 @@ my $FTPSPIDFILE=".ftps.pid";
 # invoke perl like this:
 my $perl="perl -I$srcdir";
 
-# this gets set if curl is compiled with memory debugging:
-my $memory_debug=0;
-
-# this gets set if curl is compiled with netrc debugging:
-# It has to be in the global symbol table because of the way 'requires' works
-$main::netrc_debug=0;
-my $netrc_debug = \$main::netrc_debug;
+# this gets set if curl is compiled with debugging:
+my $curl_debug=0;
 
 # name of the file that the memory debugging creates:
 my $memdump="memdump";
@@ -67,7 +62,8 @@ my $checkstunnel = &checkstunnel;
 my $ssl_version; # set if libcurl is built with SSL support
 
 my $skipped=0;  # number of tests skipped; reported in main loop
-my $problems=0; # number of tests that didn't run due to run-time problems
+my %skipped;    # skipped{reason}=counter, reasons for skip
+my @teststat;   # teststat[testnum]=reason, reasons for skip
 
 #######################################################################
 # variables the command line options may set
@@ -106,20 +102,24 @@ sub serverpid {
 # stop the given test server
 #
 sub stopserver {
-    my $PIDFILE = $_[0];
+    my $pid = $_[0];
     # check for pidfile
-    if ( -f $PIDFILE ) {
-        my $PID = serverpid($PIDFILE);
-
-        my $res = kill (9, $PID); # die!
+    if ( -f $pid ) {
+        my $PIDFILE = $pid;
+        $pid = serverpid($PIDFILE);
         unlink $PIDFILE; # server is killed
+    }
+    elsif($pid <= 0) {
+        return; # this is not a good pid
+    }
 
-        if($res && $verbose) {
-            print "RUN: Test server pid $PID signalled to die\n";
-        }
-        elsif($verbose) {
-            print "RUN: Test server pid $PID didn't exist\n";
-        }
+    my $res = kill (9, $pid); # die!
+
+    if($res && $verbose) {
+        print "RUN: Test server pid $pid signalled to die\n";
+    }
+    elsif($verbose) {
+        print "RUN: Test server pid $pid didn't exist\n";
     }
 }
 
@@ -128,24 +128,19 @@ sub stopserver {
 #
 sub checkserver {
     my ($pidfile)=@_;
-    my $RUNNING=0;
-    my $PID=0;
+    my $pid=0;
 
     # check for pidfile
     if ( -f $pidfile ) {
-        $PID=serverpid($pidfile);
-        if ($PID ne "" && kill(0, $PID)) {
-            $RUNNING=1;
+        $pid=serverpid($pidfile);
+        if ($pid ne "" && kill(0, $pid)) {
+            return $pid;
         }
         else {
-            $RUNNING=0;
-            $PID = -$PID; # negative means dead process
+            return -$pid; # negative means dead process
         }
     }
-    else {
-        $RUNNING=0;
-    }
-    return $PID
+    return 0;
 }
 
 #######################################################################
@@ -179,7 +174,7 @@ sub runhttpserver {
     }
     elsif($data) {
         print "RUN: Unknown HTTP server is running on port $HOSTPORT\n";
-        return 2;
+        return -2;
     }
 
     if($pid > 0) {
@@ -193,7 +188,7 @@ sub runhttpserver {
     }
 
     my $flag=$debugprotocol?"-v ":"";
-    my $cmd="$perl $srcdir/httpserver.pl $flag $HOSTPORT &";
+    $cmd="$perl $srcdir/httpserver.pl $flag $HOSTPORT &";
     system($cmd);
     if($verbose) {
         print "CMD: $cmd\n";
@@ -204,25 +199,26 @@ sub runhttpserver {
         # verify that our server is up and running:
         my $data=`$CURL --silent -i $HOSTIP:$HOSTPORT/verifiedserver 2>/dev/null`;
 
-        if ( $data !~ /WE ROOLZ: (\d+)/ ) {
-            sleep(1);
-            next;
-        }
-        else {
+        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
+            $pid = 0+$1;
             $verified = 1;
             last;
         }
+        else {
+            sleep(1);
+            next;
+        }
     }
     if(!$verified) {
         print STDERR "RUN: failed to start our HTTP server\n";
-        return 1;
+        return -1;
     }
 
     if($verbose) {
         print "RUN: HTTP server is now verified to be our server\n";
     }
 
-    return 0;
+    return $pid;
 }
 
 #######################################################################
@@ -232,12 +228,12 @@ sub runhttpsserver {
     my $verbose = $_[0];
     my $STATUS;
     my $RUNNING;
-    my $PID=checkserver($HTTPSPIDFILE );
+    my $pid=checkserver($HTTPSPIDFILE );
 
-    if($PID > 0) {
+    if($pid > 0) {
         # kill previous stunnel!
         if($verbose) {
-            print "RUN: kills off running stunnel at $PID\n";
+            print "RUN: kills off running stunnel at $pid\n";
         }
         stopserver($HTTPSPIDFILE);
     }
@@ -249,6 +245,9 @@ sub runhttpsserver {
         print "CMD: $cmd\n";
     }
     sleep(1);
+    $pid=checkserver($HTTPSPIDFILE);
+
+    return $pid;
 }
 
 #######################################################################
@@ -275,13 +274,13 @@ sub runftpserver {
         
         if ( $data =~ /WE ROOLZ: (\d+)/ ) {
             # this is our test server with a known pid!
-            $pid = $1;
+            $pid = 0+$1;
         }
         else {
             if($data || ($took > 2)) {
                 # this is not a known server
                 print "RUN: Unknown server on our favourite port: $FTPPORT\n";
-                return 1;
+                return -1;
             }
         }
     }
@@ -292,7 +291,7 @@ sub runftpserver {
         if(!$res) {
             print "RUN: Failed to kill our FTP test server, do it manually and",
             " restart the tests.\n";
-            exit;
+            return -1;
         }
         sleep(1);
     }
@@ -310,27 +309,29 @@ sub runftpserver {
         # verify that our server is up and running:
         my $data=`$CURL --silent -i ftp://$HOSTIP:$FTPPORT/verifiedserver 2>/dev/null`;
 
-        if ( $data !~ /WE ROOLZ: (\d+)/ ) {
+        if ( $data =~ /WE ROOLZ: (\d+)/ ) {
+            $pid = 0+$1;
+            $verified = 1;
+            last;
+        }
+        else {
             if($verbose) {
                 print STDERR "RUN: Retrying FTP server existance in 1 sec\n";
             }
             sleep(1);
             next;
         }
-        else {
-            $verified = 1;
-            last;
-        }
     }
     if(!$verified) {
-        die "RUN: failed to start our FTP server\n";
+        warn "RUN: failed to start our FTP server\n";
+        return -2;
     }
 
     if($verbose) {
         print "RUN: FTP server is now verified to be our server\n";
     }
 
-    return 0;
+    return $pid;
 }
 
 #######################################################################
@@ -340,12 +341,12 @@ sub runftpsserver {
     my $verbose = $_[0];
     my $STATUS;
     my $RUNNING;
-    my $PID=checkserver($FTPSPIDFILE );
+    my $pid=checkserver($FTPSPIDFILE );
 
-    if($PID > 0) {
+    if($pid > 0) {
         # kill previous stunnel!
         if($verbose) {
-            print "kills off running stunnel at $PID\n";
+            print "kills off running stunnel at $pid\n";
         }
         stopserver($FTPSPIDFILE);
     }
@@ -357,6 +358,10 @@ sub runftpsserver {
         print "CMD: $cmd\n";
     }
     sleep(1);
+
+    $pid=checkserver($FTPSPIDFILE );
+
+    return $pid;
 }
 
 #######################################################################
@@ -432,17 +437,56 @@ sub compare {
 #######################################################################
 # display information about curl and the host the test suite runs on
 #
-sub displaydata {
+sub checkcurl {
 
     unlink($memdump); # remove this if there was one left
 
-    my $version=`$CURL -V`;
-    chomp $version;
+    my $curl;
+    my $libcurl;
+    my @version=`$CURL -V`;
+    for(@version) {
+        chomp;
 
-    my $curl = $version;
+        if($_ =~ /^curl/) {
+            $curl = $_;
 
-    $curl =~ s/^(.*)(libcurl.*)/$1/g;
-    my $libcurl = $2;
+            $curl =~ s/^(.*)(libcurl.*)/$1/g;
+            $libcurl = $2;
+
+           if ($curl =~ /win32/)
+           {
+               # Native Windows builds don't understand the
+               # output of cygwin's pwd.  It will be
+               # something like /cygdrive/c/<some path>.
+               #
+               # Use the cygpath utility to convert the
+               # working directory to a Windows friendly
+               # path.  The -m option converts to use drive
+               # letter:, but it uses / instead \.  Forward
+               # slashes (/) are easier for us.  We don't
+               # have to escape them to get them to curl
+               # through a shell.
+               chomp($pwd = `cygpath -m $pwd`);
+           }
+        }
+        elsif($_ =~ /^Protocols: (.*)/i) {
+            # these are the supported protocols, we don't use this knowledge
+            # at this point
+        }
+        elsif($_ =~ /^Features: (.*)/i) {
+            my $feat = $1;
+            if($feat =~ /debug/i) {
+                # debug is a listed "feature", use that knowledge
+                $curl_debug = 1;
+                # set the NETRC debug env
+                $ENV{'CURL_DEBUG_NETRC'} = 'log/netrc';
+            }
+            if($feat =~ /SSL/i) {
+                # ssl enabled
+                $ssl_version=1;
+            }
+        }
+    }
 
     my $hostname=`hostname`;
     my $hosttype=`uname -a`;
@@ -453,26 +497,9 @@ sub displaydata {
     "* Host: $hostname",
     "* System: $hosttype";
 
-    if($libcurl =~ /SSL/i) {
-        $ssl_version=1;
-    }
-
-    if( -r $memdump) {
-        # if this exists, curl was compiled with memory debugging
-        # enabled and we shall verify that no memory leaks exist
-        # after each and every test!
-        $memory_debug=1;
-
-        # there's only one debug control in the configure script
-        # so hope netrc debugging is enabled and set it up
-        $$netrc_debug = 1;
-        $ENV{'CURL_DEBUG_NETRC'} = 'log/netrc';
-    }
-    printf("* Memory debugging: %s\n", $memory_debug?"ON":"OFF");
-    printf("* Netrc debugging:  %s\n", $$netrc_debug?"ON":"OFF");
-    printf("* HTTPS server:     %s\n", $checkstunnel?"ON":"OFF");
-    printf("* FTPS server:      %s\n", $checkstunnel?"ON":"OFF");
+    printf("* Server SSL:       %s\n", $checkstunnel?"ON":"OFF");
     printf("* libcurl SSL:      %s\n", $ssl_version?"ON":"OFF");
+    printf("* libcurl debug:    %s\n", $curl_debug?"ON":"OFF");
     print "***************************************** \n";
 }
 
@@ -498,51 +525,81 @@ sub subVariables {
 sub singletest {
     my $testnum=$_[0];
 
+    my @what;
+    my $why;
+    my $serverproblem;
+
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
         if($verbose) {
             # this is not a test
             print "RUN: $testnum doesn't look like a test case!\n";
         }
-        return -1;
+        $serverproblem = 100;
+    }
+    else {
+        @what = getpart("client", "features");
     }
 
-    my $serverproblem = serverfortest($testnum);
+    printf("test %03d...", $testnum);
+    
+    for(@what) {
+        my $f = $_;
+        $f =~ s/\s//g;
+
+        if($f eq "SSL") {
+            if($ssl_version) {
+                next;
+            }
+        }
+        elsif($f eq "netrc_debug") {
+            if($curl_debug) {
+                next;
+            }
+        }
+
+        $why = "lacks $f";
+        $serverproblem = 15; # set it here
+        last;
+    }
+
+    if(!$serverproblem) {
+        $serverproblem = serverfortest($testnum);
+    }
 
     if($serverproblem) {
         # there's a problem with the server, don't run
         # this particular server, but count it as "skipped"
-        if($serverproblem> 1) {
-            print "RUN: test case $testnum couldn't run!\n";
-            $problems++;
+        if($serverproblem == 2) {
+            $why = "server problems";
+        }
+        elsif($serverproblem == 100) {
+            $why = "no test";
+        }
+        elsif($serverproblem == 99) {
+            $why = "bad test";
+        }
+        elsif($serverproblem == 15) {
+            # set above, a lacking prereq
+        }
+        elsif($serverproblem == 1) {
+            $why = "no SSL-capable server";
         }
         else {
-            $skipped++;
+            $why = "unfulfilled requirements";
         }
+        $skipped++;
+        $skipped{$why}++;
+        $teststat[$testnum]=$why; # store reason for this test case
+        
+        print "SKIPPED\n";
+        if(!$short) {
+            print "* Test $testnum: $why\n";
+        }
+
         return -1;
     }
 
-    {
-        my %hash = getpartattr("client");
-        my $requires = $hash{'requires'};
-
-        if (defined($requires)) {
-            no strict "refs";
-            my $value=${$requires};
-#            print "This test requires '$requires' with value '$value' \n";
-
-            if (${$requires}) {
-                # this test is OK
-                ;
-            }else {
-                print "$testnum requires $requires, which is not set; skipping\n";
-                $skipped++;
-                return -1;  # return test-not-run
-            }
-        }
-    }
-
-
     # extract the reply data
     my @reply = getpart("reply", "data");
     my @replycheck = getpart("reply", "datacheck");
@@ -584,7 +641,6 @@ sub singletest {
     # name of the test
     my @testname= getpart("client", "name");
 
-    printf("test %03d...", $testnum);
     if(!$short) {
         my $name = $testname[0];
         $name =~ s/\n//g;
@@ -611,25 +667,33 @@ sub singletest {
         writearray($FTPDCMD, \@ftpservercmd);
     }
 
+    my (@setenv)= getpart("client", "setenv");
+    my @envs;
+
+    my $s;
+    for $s (@setenv) {
+        chomp $s; # cut off the newline
+
+        subVariables \$s;
+
+        if($s =~ /([^=]*)=(.*)/) {
+            my ($var, $content)=($1, $2);
+            $ENV{$var}=$content;
+            # remember which, so that we can clear them afterwards!
+            push @envs, $var;
+        }
+    }
+
     # get the command line options to use
     my ($cmd, @blaha)= getpart("client", "command");
 
     # make some nice replace operations
     $cmd =~ s/\n//g; # no newlines please
 
+    # substitute variables in the command line
     subVariables \$cmd;
 
-#    $cmd =~ s/%HOSTIP/$HOSTIP/g;
-#    $cmd =~ s/%HOSTPORT/$HOSTPORT/g;
-#    $cmd =~ s/%HTTPSPORT/$HTTPSPORT/g;
-#    $cmd =~ s/%FTPPORT/$FTPPORT/g;
-#    $cmd =~ s/%FTPSPORT/$FTPSPORT/g;
-#    $cmd =~ s/%SRCDIR/$srcdir/g;
-#    $cmd =~ s/%PWD/$pwd/g;
-
-    #$cmd =~ s/%HOSTNAME/$HOSTNAME/g;
-
-    if($memory_debug) {
+    if($curl_debug) {
         unlink($memdump);
     }
 
@@ -657,10 +721,8 @@ sub singletest {
 
     my $out="";
 
-    if($cmdhash{'option'} eq "no-output") {
-        #print "*** We don't slap on --output\n";
-    }
-    else {
+    if($cmdhash{'option'} !~ /no-output/) {
+        #We may slap on --output!
         if (!@validstdout) {
             $out=" --output $CURLOUT ";
         }
@@ -694,7 +756,7 @@ sub singletest {
         $DBGCURL=$CMDLINE;
     }
 
-    $CMDLINE .= "$cmdargs >$STDOUT 2>$STDERR";
+    $CMDLINE .= "$cmdargs >>$STDOUT 2>>$STDERR";
 
     if($verbose) {
         print "$CMDLINE\n"; 
@@ -720,6 +782,11 @@ sub singletest {
     # remove the special FTP command file after each test!
     unlink($FTPDCMD);
 
+    my $e;
+    for $e (@envs) {
+        $ENV{$e}=""; # clean up
+    }
+
     my @err = getpart("verify", "errorcode");
     my $errorcode = $err[0];
 
@@ -845,20 +912,20 @@ sub singletest {
 
     unlink($FTPDCMD); # remove the instructions for this test
 
-    my @what = getpart("client", "killserver");
+    @what = getpart("client", "killserver");
     for(@what) {
         my $serv = $_;
         chomp $serv;
         if($run{$serv}) {
             stopserver($run{$serv}); # the pid file is in the hash table
-            $run{$serv}=""; # clear it
+            $run{$serv}=0; # clear pid
         }
         else {
             print STDERR "RUN: The $serv server is not running\n";
         }
     }
 
-    if($memory_debug) {
+    if($curl_debug) {
         if(! -f $memdump) {
             print "\n** ALERT! memory debuggin without any output file?\n";
         }
@@ -896,9 +963,16 @@ sub singletest {
 # This function makes sure the right set of server is running for the
 # specified test case. This is a useful design when we run single tests as not
 # all servers need to run then!
+#
+# Returns:
+# 100 if this is not a test case
+# 99  if this test case has no servers specified
+# 2   if one of the required servers couldn't be started
+# 1   if this test is skipped due to unfulfilled SSL/stunnel-requirements
 
 sub serverfortest {
     my ($testnum)=@_;
+    my $pid;
 
     # load the test case file definition
     if(loadtest("${TESTDIR}/test${testnum}")) {
@@ -908,11 +982,12 @@ sub serverfortest {
         }
         return 100;
     }
+
     my @what = getpart("client", "server");
 
     if(!$what[0]) {
         warn "Test case $testnum has no server(s) specified!";
-        return 100;
+        return 99;
     }
 
     for(@what) {
@@ -920,18 +995,20 @@ sub serverfortest {
         $what =~ s/[^a-z]//g;
         if($what eq "ftp") {
             if(!$run{'ftp'}) {
-                if(runftpserver($verbose)) {
+                $pid = runftpserver($verbose);
+                if($pid <= 0) {
                     return 2; # error starting it
                 }
-                $run{'ftp'}=$FTPPIDFILE;
+                $run{'ftp'}=$pid;
             }
         }
         elsif($what eq "http") {
             if(!$run{'http'}) {
-                if(runhttpserver($verbose)) {
+                $pid = runhttpserver($verbose);
+                if($pid <= 0) {
                     return 2; # error starting
                 }
-                $run{'http'}=$HTTPPIDFILE;
+                $run{'http'}=$pid;
             }
         }
         elsif($what eq "ftps") {
@@ -941,14 +1018,18 @@ sub serverfortest {
                 return 1;
             }
             if(!$run{'ftp'}) {
-                if(runftpserver($verbose)) {
+                $pid = runftpserver($verbose);
+                if($pid <= 0) {
                     return 2; # error starting it
                 }
-                $run{'ftp'}=$FTPPIDFILE;
+                $run{'ftp'}=$pid;
             }
             if(!$run{'ftps'}) {
-                runftpsserver($verbose);
-                $run{'ftps'}=$FTPSPIDFILE;
+                $pid = runftpsserver($verbose);
+                if($pid <= 0) {
+                    return 2;
+                }
+                $run{'ftps'}=$pid;
             }
         }
         elsif($what eq "file") {
@@ -961,14 +1042,18 @@ sub serverfortest {
                 return 1;
             }
             if(!$run{'http'}) {
-                if(runhttpserver($verbose)) {
+                $pid = runhttpserver($verbose);
+                if($pid <= 0) {
                     return 2; # problems starting server
                 }
-                $run{'http'}=$HTTPPIDFILE;
+                $run{'http'}=$pid;
             }
             if(!$run{'https'}) {
-                runhttpsserver($verbose);
-                $run{'https'}=$HTTPSPIDFILE;
+                $pid = runhttpsserver($verbose);
+                if($pid <= 0) {
+                    return 2;
+                }
+                $run{'https'}=$pid;
             }
         }
         elsif($what eq "none") {
@@ -977,7 +1062,6 @@ sub serverfortest {
             warn "we don't support a server for $what";
         }
     }
-    return 0; # ok
 }
 
 #######################################################################
@@ -1064,7 +1148,7 @@ if($testthis[0] ne "") {
 #
 
 if(!$listonly) {
-    displaydata();
+    checkcurl();
 }
 
 #######################################################################
@@ -1109,11 +1193,14 @@ my $failed;
 my $testnum;
 my $ok=0;
 my $total=0;
+my $lasttest;
 
 foreach $testnum (split(" ", $TESTCASES)) {
 
+    $lasttest = $testnum if($testnum > $lasttest);
+
     my $error = singletest($testnum);
-    if(-1 == $error) {
+    if($error < 0) {
         # not a test we can run
         next;
     }
@@ -1144,10 +1231,14 @@ close(CMDLOG);
 # Tests done, stop the servers
 #
 
+print "Shutting down test suite servers:\n" if (!$short);
 for(keys %run) {
+    printf ("* kill pid for %-5s => %-5d\n", $_, $run{$_}) if(!$short);
     stopserver($run{$_}); # the pid file is in the hash table
 }
 
+my $all = $total + $skipped;
+
 if($total) {
     printf("TESTDONE: $ok tests out of $total reported OK: %d%%\n",
            $ok/$total*100);
@@ -1159,11 +1250,32 @@ if($total) {
 else {
     print "TESTFAIL: No tests were performed!\n";
 }
-if($skipped) {
-    print "TESTINFO: $skipped tests were skipped due to restraints\n";
+
+if($all) {
+    print "TESTDONE: $all tests were considered.\n";
 }
-if($problems) {
-    print "TESTINFO: $problems tests didn't run due to run-time problems\n";
+
+if($skipped) {
+    my $s=0;
+    print "TESTINFO: $skipped tests were skipped due to these restraints:\n";
+
+    for(keys %skipped) {
+        my $r = $_;
+        printf "TESTINFO: \"%s\" %d times (", $r, $skipped{$_};
+
+        # now show all test case numbers that had this reason for being
+        # skipped
+        my $c=0;
+        for(0 .. $lasttest) {
+            my $t = $_;
+            if($teststat[$_] eq $r) {
+                print ", " if($c);
+                print $_;
+                $c++;
+            }
+        }
+        print ")\n";
+    }
 }
 if($total && ($ok != $total)) {
     exit 1;
